#!/bin/sh
cat <<%EOF%
#############################################################
#                                                           #
#                           SCAN                            #
#                         Yingli Niu                        #
#                         2015-04-07                        #
#                                                           #
#############################################################
%EOF%
#===============================================================================
function StartJob(){
	#----------------------------------------------------------------------------
	if [ -f finished ]; then
		echo
		echo "The job has been done!"
		echo
		exit 0
	fi
	#----------------------------------------------------------------------------
	WORKDIR=`pwd`
	PESDIR=$WORKDIR/PES
	ncpu_all=1
	ncore=1
	remoteshell="ssh"
	machinefile=""
	task="none"
	tmpfile="temp"
	tmpfile2="temp2"
	LANG="C"
	LC_ALL="C"
	LC_CTYPE="C"
	HOST=`hostname`
	save_chk="no"
	lock="lock"
	stop="stop"
	recalc="no"
	mpipath="none"
	test_rs="no"
	save_fchk="no"
	chkfile="none"
	search_chk="no"
	ignore_log_err="yes"
	GetCoordProgram="none"
	GetEnergyProgram="none"
	coord="yes"
	scan="yes"
	energy="yes"
	checkpara="yes"
	#----------------------------------------------------------------------------
	echo
	echo "Start Job"
	date '+Time: %Y-%m-%d %T'
	echo
	echo " running on $HOST"
	echo " date: "`date '+Time: %Y-%m-%d %T'`
	echo
}
#
#===============================================================================
#
function CheckArguments(){
	#----------------------------------------------------------------------------
	# Get Program directory
	#
	scriptdir=$(dirname $0)
	#----------------------------------------------------------------------------
	# check lock file
	#
	nnn=`echo $lock | grep lock | wc -l`
	if [ $nnn -eq 0 ]; then
		lock=lock_$lock
	fi
	#----------------------------------------------------------------------------
	# check ncore
	#
	if [ $ncore -lt 1 ]; then
		$ncore = 1
	fi
	
	inicoord=`grep inicoord $ifile | cut -d "=" -f 2 | sed 's/\"//g'`
	scantype=`grep scantype $ifile | cut -d "=" -f 2 | sed 's/\"//g'`
	if [ ! -z $inicoord ]; then
		ii=$inicoord
	elif [ ${scantype}x = "manualx" ]; then
		ii=$scantype
	else
		ii=`grep imode $ifile | cut -d "=" -f 2`
		ii=`expr $ii + 0`
	fi
	PESDIR=$WORKDIR/PES-$ii
	PESDIR=${PESDIR/" "/}
	PESDIR=`echo $PESDIR | sed s/[[:space:]]//g`
	cpu_all=`cat $machinefile | wc -l`
	
	echo "There are $cpu_all processors!"
	
	[ -z $ifile -o ! -f $ifile ] && echo "input file !!!" && exit 0
	
	if [ $cpu_all -gt 1 ]; then
	if [ -z $machinefile ]; then
		echo
		echo "Nodes file not be found!"
		echo "Stop!"
		echo
		exit 0
	fi
	fi
	[ "$cpu_all" -gt "1" -a ! -f $machinefile ] && echo "nodes file!!!" && exit 0
	############################################################
	numfreq_single=$scriptdir/numfreq_single.sh
	############################################################
	echo PESDIR is : $PESDIR
	echo
	if [ ! -d $PESDIR ]; then
	mkdir $PESDIR
	fi

	#============================================================================
	echo "--------------------------------------------------------------"
	echo "Parameters"
	echo "--------------------------------------------------------------"
	echo "qctype         = $qctype"
	echo "qcpath         = $qcpath"
	echo "mpipath        = $mpipath"
	echo "ex             = $ex"
	echo "nodefile       = $machinefile"
	echo "input          = $ifile"
	echo "scratch        = $scratch"
	echo "recalc         = $recalc"
	echo "task           = $task"
	echo "coord          = $coord"
	echo "scan           = $scan"
	echo "energy         = $energy"
	echo "heckpara       = $checkpara"
	echo "save_chk       = $save_chk"
	echo "save_fchk      = $save_fchk"
	echo "ncore          = $ncore"
	echo "test_rs        = $test_rs"
	echo "chkfile        = $chkfile"
	echo "search_chk     = $search_chk"
	echo "ignore_log_err = $ignore_log_err"
	echo "lock           = $lock"
	echo "stop           = $stop"
	echo "--------------------------------------------------------------"


}
#
#===============================================================================
#
function GenerateCoord(){
	if [ ${qctype}x = "gaussianx" ]; then
		if [ ${task}x = "nonex" ]; then
			if [ ! -z $inicoord ]; then
				GetCoordProgram="$scriptdir/gaussian_pes_intcoord $ifile"
			elif [ ${scantype}x = "manualx" ]; then
				GetCoordProgram="$scriptdir/gaussian_pes_mancoord $ifile"
			fi
		elif [ ${task}x = "modex" ]; then
			GetCoordProgram="$scriptdir/gaussian_pes_mode_coord $ifile"
		elif [ ${task}x = "numfreqx" ]; then
			GetCoordProgram="$scriptdir/numfreq_coord.exe $ifile"
		elif [ ${task}x = "pes2x" ]; then
			GetCoordProgram="$scriptdir/gaussian_pes2_coord $ifile"
		elif [ ${task}x = "intx" ]; then
			GetCoordProgram="$scriptdir/gaussian_pes_int_coord $ifile"
		elif [ ${task}x = "afmx" ]; then
			GetCoordProgram="$scriptdir/scan_afm_coord ${qctype} $ifile"
		elif [ ${task}x = "clusterx" ]; then
			GetCoordProgram="$scriptdir/gaussian_cluster_coord $ifile"
		else
			echo "Input error, exit!"
			exit 1
		fi
	elif [ ${qctype}x = "turbomolex" ]; then
		if [ ${task}x = "numfx" ]; then
			GetCoordProgram="$scriptdir/numfreq_coord ${qctype} $ifile"
		fi
	elif [ ${qctype}x = "molprox" ]; then
		GetCoordProgram="$scriptdir/scan_mode_coord ${qctype} $ifile"
	elif [ ${qctype}x = "vaspx" ]; then
		if [ ${task}x = "afmx" ]; then
			GetCoordProgram="$scriptdir/scan_afm_coord $ifile"
		else
			GetCoordProgram="$scriptdir/vasp_coord $ifile"
		fi
	elif [ ${qctype}x = "nwchemx" ]; then
		GetCoordProgram="$scriptdir/scan_afm_coord $ifile"
	elif [ ${qctype}x = "pwscfx" ]; then
		GetCoordProgram="$scriptdir/pwscf_coord $ifile"
	elif [ ${qctype}x = "wien2kx" ]; then
		if [ ${task}x = "densityx" ]; then
			GetCoordProgram="$scriptdir/wien2k_coord $ifile"
		else
			echo "Error: task ${task} is not implemented!"
			exit
		fi
	else
		echo "Error: ${qctype} is not implemented!"
		exit
	fi
	echo "cd $WORKDIR"
	cd $WORKDIR
	echo "Generate coordinate file:"
	echo "$GetCoordProgram"
	$GetCoordProgram
}
#
#===============================================================================
#
function GenerateEnergy(){
	cd $PESDIR
	if [ ${qctype}x = "gaussianx" ]; then
		if [ ${energy}x != "nox" ]; then
			if [ ${task}x = "nonex" ]; then
				GetEnergyProgram="$scriptdir/gaussian_pes_energy"
			elif [ ${task}x = "modex" ]; then
				GetEnergyProgram="$scriptdir/gaussian_pes_mode_energy"
			elif [ ${task}x = "numfreqx" ]; then
				GetEnergyProgram="$scriptdir/numfreq_gather.exe ../$ifile"
			elif [ ${task}x = "pes2x" ]; then
				GetEnergyProgram="$scriptdir/gaussian_pes2_energy"
			elif [ ${task}x = "intx" ]; then
				GetEnergyProgram="$scriptdir/gaussian_pes_int_energy"
			elif [ ${task}x = "clusterx" ]; then
				GetEnergyProgram="$scriptdir/gaussian_pes_int_energy"
			elif [ ${task}x = "afmx" ]; then
				GetEnergyProgram="$scriptdir/numfreq_gather.exe"
			else
				echo "Input error, exit!"
				exit
			fi
		fi
	elif [ ${qctype}x = "turbomolex" ]; then
		if [ ${task}x = "numfx" ]; then
			GetCoordProgram="$scriptdir/scan_numf_energy $ifile"
		fi
	elif [ ${qctype}x = "molprox" ]; then
		GetEnergyProgram="$scriptdir/scan_mode_energy"
	elif [ ${qctype}x = "vaspx" ]; then
		if [ ${task}x = "afmx" ]; then
			GetEnergyProgram="$scriptdir/scan_afm_energy"
		else
			GetEnergyProgram="$scriptdir/vasp_energy"
		fi
	elif [ ${qctype}x = "pwscfx" ]; then
		GetEnergyProgram="$scriptdir/pwscf_energy"
	elif [ ${qctype}x = "nwchemx" ]; then
		GetEnergyProgram="$scriptdir/scan_afm_energy"
	elif [ ${qctype}x = "wien2kx" ]; then
		GetEnergyProgram="$scriptdir/wien2k_energy"
	fi
	echo "cd $PESDIR"
	cd $PESDIR
	echo "Generate scan information:"
	echo "$GetEnergyProgram"
	$GetEnergyProgram
}
#
#===============================================================================
#
function CheckParallel() {
	if [ -f $machinefile ]; then
		nmach=`     cat  $machinefile | wc -l`
		nmach_uniq=`uniq $machinefile | wc -l`
		#
		RAWHOSTLIST=`     cat  $machinefile`
		RAWHOSTLIST_uniq=`uniq $machinefile`
		#
		cpu_all=""
		cpu_uniq=""
		cpu_calc=""
		ncpu_all=0
		ncpu_calc=0
		for cpu in $RAWHOSTLIST_uniq; do
			ncpu_node=`grep $cpu $machinefile | wc -l`
			node=`     echo $cpu | cut -f 1 -d "."`
			#
			if [ $node != $HOST ]; then
				$remoteshell $node "echo login-TEST on $machine"
				echo $node $?
				if [ $? -ne 0 ]; then
					echo
					echo "Error logging in on $machine"
					echo "Check your machine-file $machfile"
					exit 1
				fi
			else
				echo $machine is local host
			fi
			#
			cpu_uniq="$cpu_uniq $node"
			icpu_node=0
			icpu_calc=0
			while [ $icpu_node -lt $ncpu_node ]; do
				ncpu_mod=`echo $icpu_node % $ncore | bc`
				if [ $ncpu_mod -eq 0 ]; then
					icpu_calc=`expr $icpu_calc + 1`
					ncpu_calc=`expr $ncpu_calc + 1`
					i2=`echo ${icpu_calc} | awk '{printf("%02d\n",$0)}'`
					cpu_calc="$cpu_calc ${node}#$i2"
				fi
				#-------------------------------------------------------------------
				icpu_node=`expr $icpu_node + 1`
				ncpu_all=` expr $ncpu_all  + 1`
				i2=`echo ${icpu_node} | awk '{printf("%02d\n",$0)}'`
				cpu_all="$cpu_all ${node}#$i2"
			done
			#
		done
		#
		echo "machinefile is $machinefile"
		echo "-----------------------------"
		cat $machinefile
		echo "-----------------------------"
		echo "CPUSTEP" $ncore
		echo "HOSTs:"
		echo $cpu_calc
		echo "ncpu_calc=${ncpu_calc}"
		echo
		echo "scratch_dir: $scratch"
		echo
		for nodename in $cpu_uniq; do
			echo "nodename: $nodename"
		done
		echo
	else
		echo; echo "Can't find machine-file $machinefile"
		exit 1;
	fi
}
#
#===============================================================================
#
function DoSCAN() {
	cd $PESDIR
	#
	NDir=`grep "coord" files.out | wc -l`
	dirs=`grep "coord" files.out        `
	i=1
	init="yes"
	for dir in $dirs; do
		if [ -f $stop -a ${init}x = "yesx" ]; then
			echo "FOUND stop FILE. TERMINATING ... "
			exit 0
		fi
		#-------------------------------------------------------------------------
		init="no"
		if_calc="yes"
		#-------------------------------------------------------------------------
		if [ -s $dir.log ]; then
			if_calc="no"
			#----------------------------------------------------------------------
			if [ ${qctype}x = "gaussianx" ]; then
				num_Normal=`grep "Normal termination" $dir.log 2> /dev/null | wc -l`
				if [ ${num_Normal} -eq 0 ]; then
					if_calc="yes"
				fi
				#
				if [ ${ignore_log_err}x = "yesx" ]; then
					if_calc="no"
				fi
			elif [ ${qctype}x = "molprox" ]; then
				num_Normal=`grep "Variable memory released" $dir.log 2> /dev/null | wc -l`
				if [ ${num_Normal} -eq 0 ]; then
					if_calc="yes"
				fi
				#
				if [ ${ignore_log_err}x = "yesx" ]; then
					if_calc="no"
				fi
			elif [ ${qctype}x = "turbomolex" ]; then
				num_Normal=`grep "all done" $dir.log 2> /dev/null | wc -l`
				if [ ${num_Normal} -eq 0 ]; then
					if_calc="yes"
				fi
				#
				if [ ${ignore_log_err}x = "yesx" ]; then
					if_calc="no"
				fi
			elif [ ${qctype}x = "nwchemx" ]; then
				num_Normal=`grep "GA Statistics" $dir.log 2> /dev/null | wc -l`
				if [ ${num_Normal} -eq 0 ]; then
					if_calc="yes"
				fi
				#
				if [ ${ignore_log_err}x = "yesx" ]; then
					if_calc="no"
				fi
			elif [ ${qctype}x = "vaspx" ]; then
				num_Normal=`grep "General timing" $dir.log 2> /dev/null | wc -l`
				if [ ${num_Normal} -eq 0 ]; then
					if_calc="yes"
				fi
				#
				if [ ${ignore_log_err}x = "yesx" ]; then
					if_calc="no"
				fi
			fi
			#----------------------------------------------------------------------
		fi
		#-------------------------------------------------------------------------
		if [ ${recalc}x = "yesx" ]; then
			if_calc="yes"
		fi
		#-------------------------------------------------------------------------
		if [ ${if_calc}x = "nox" ]; then
			if [ ! -f $stop ]; then
				echo "step $dir was already done (# $i of $NDir)"
			fi
		else
			if [ ! -f $stop ]; then
				echo "step $dir started (# $i of $NDir)"
				arguments="	$dir			\
								$WORKDIR		\
								$PESDIR		\
								$scratch		\
								$qcpath		\
								$ncore		\
								$save_chk	\
								$save_fchk	\
								$chkfile    \
								$qctype		\
								$mpipath    \
								$search_chk \
								$task"
				if [ $ncpu_all -eq 1 ]; then
					#----------------------------------------------------------------------
					# non-parallel runs
					#
					machine=$HOSTNAME
					n1=`ls lock* 2> /dev/null | grep "$machine" | wc -l`
					n2=`ls lock* 2> /dev/null | grep "$dir"     | wc -l`
					nn=$[ $n1 + $n2 ]
					if [ $nn -eq 0 -a ${test_rs}x = "nox" ]; then
						lockfile=$lock.$machine.$dir
						touch $lockfile
						echo "----------"
						echo "numfreq_single.sh at $machine"
						$numfreq_single $lockfile $arguments
					fi
				else
					#----------------------------------------------------------------------
					# look for a free host
					#
					for machine in $cpu_calc; do
						n1=`ls lock* 2> /dev/null | grep "$machine" | wc -l`
						n2=`ls lock* 2> /dev/null | grep "$dir"     | wc -l`
						nn=$[ $n1 + $n2 ]
						if [ $nn -eq 0 -a ${test_rs}x = "nox" ]; then
							node=`echo $machine | cut -f 1 -d "#"`
							lockfile=$lock.$machine.$dir
							touch $lockfile
							if [ ${node}x = ${HOST}x ]; then
								echo "----------"
								echo "numfreq_single.sh at $machine"
								$numfreq_single $lockfile $arguments &
							else
								echo "----------"
								echo "numfreq_single.sh at $machine"
								$remoteshell $node "$numfreq_single $lockfile $arguments" &
							fi
						fi
					done
				fi
			fi
			# wait until there are free hosts
			while [ `ls -l $lock.* 2>/dev/null | wc -l` -ge $ncpu_calc ]; do
				sleep 1
			done
		fi
		# check whether a stop file exists
		if [ -f $stop ]; then
			ncpu_calc=`echo $ncpu_calc - 1 | bc`
			if [ $ncpu_calc -eq 0 ]; then
				rm -f $stop
				echo "FOUND stop FILE. TERMINATING..."
				exit 0
			fi
		fi
		i=`expr $i + 1`
	done
	wait
	sleep 1
}
#
#==============================================================================#
################################################################################
##############################              ####################################
############################## MAIN PROGRAM ####################################
##############################              ####################################
################################################################################
#==============================================================================#
# 1 StartJob
#
StartJob
#===============================================================================
# 2 Get Arguments
#
while [ $# -gt 0 ]; do
  case "$1" in
    -qctype         )          qctype="$2"  ; shift; shift ;;
    -qcpath         )          qcpath="$2"  ; shift; shift ;;
    -mpipath        )         mpipath="$2"  ; shift; shift ;;
    -ex             )              ex="$2"  ; shift; shift ;;
    -nodefile       )     machinefile="$2"  ; shift; shift ;;
    -input          )           ifile="$2"  ; shift; shift ;;
    -scratch        )         scratch="$2"  ; shift; shift ;;
    -recalc         )          recalc="$2"  ; shift; shift ;;
    -task           )            task="$2"  ; shift; shift ;;
    -coord          )           coord="$2"  ; shift; shift ;;
    -scan           )            scan="$2"  ; shift; shift ;;
    -energy         )          energy="$2"  ; shift; shift ;;
    -checkpara      )       checkpara="$2"  ; shift; shift ;;
    -save_chk       )        save_chk="$2"  ; shift; shift ;;
    -save_fchk      )       save_fchk="$2"  ; shift; shift ;;
    -ncore          )           ncore="$2"  ; shift; shift ;;
    -test_rs        )         test_rs="$2"  ; shift; shift ;;
    -chkfile        )         chkfile="$2"  ; shift; shift ;;
    -search_chk     )      search_chk="$2"  ; shift; shift ;;
    -ignore_log_err )  ignore_log_err="$2"  ; shift; shift ;;
    -lock           )            lock="$2"  ; shift; shift ;;
    -stop           )            stop="$2"  ; shift; shift ;;
    *               )     wrong_param="$1"  ;cat <<END   ;
 echo "Wrong parameter: $wrong_param"
 USAGES:

 gaussian_pes.sh               \\
    -qctype         "qctype"   \\
    -qcpath         "qcpath"   \\
    -mpipath        "mpipath"  \\
    -ex             "ex"       \\
    -nodefile       "nodefile" \\
    -input          "input"    \\
    -scratch        "scratch"  \\
    -recalc         "yes"      \\
    -task           "numfreq" \\
    -coord          "yes"      \\
    -scan           "yes"      \\
    -energy         "yes"      \\
    -checkpara      "yes"      \\
    -save_chk       "yes"      \\
    -save_fchk      "yes"      \\
    -ncore          "8"        \\
    -test_rs        "no"       \\
    -chkfile        "none"     \\
    -search_chk     "no"       \\
    -ignore_log_err "yes"      \\
    -lock           "lock"     \\
    -stop           "stop"     \\

 parameters:

    -qctype         "gaussian"                  # gaussian or vasp
    -qcpath         "/home/gaussian/g09/g90"    # excutable quantum chemistry program
    -mpipath        "/opt/mpich2-1.5"           # mpich path
    -ex             1                           # the number of excited state
    -nodefile       "$PBS_NODEFILE"             # nodefile
    -input          "input"                     # input file
    -scratch        "/scratch/$USER"            # scratch directory
    -task           "numfreq"                  # numerical calcualation of hessian matrix
                    "int"                       # scan internal coodinates
                    "afm"                       # scan afm map
                    "density"                   # calculate density for WIEN2K
    -coord          "yes" or "no"               # default "yes"
    -scan           "yes" or "no"               # default "yes"
    -energy         "yes" or "no"               # default "yes"
    -checkpara      "yes" or "no"               # default "yes"
    -save_chk       "yes"                       # if save chk file
    -save_fchk      "yes"                       # if save fchk file
    -ncore          "n"                         # default "1"
    -test_rs        "no"                        # default "no"
    -chkfile        "none"                      # defualt "none"
    -search_chk     "no"                        # defualt "no", if search the nearest chk file
    -ignore_log_err "yes"                       # default "yes"
    -lock           "lock_name"                 # default "lock"
    -stop           "stop_name"                 # default "stop"
END
    exit;;
  esac
done
#===============================================================================
# 3 Check Arguments
#
CheckArguments
#===============================================================================
# 4 Generate coord files
#
#-------------------------------------------------------------------------------
if [ ${coord}x = "yesx" ]; then
	cd $WORKDIR
	GenerateCoord
fi
#===============================================================================
# 5 Check Parallel
#
if [ ${checkpara}x = "yesx" ]; then
	CheckParallel
	echo
	echo "CheckParallel Done !!!"
	date '+Time: %Y-%m-%d %T'
	echo
fi
#===============================================================================
# 6 Do SCAN
#
if [ ${scan}x = "yesx" ]; then
	DoSCAN
	echo
	echo "Scaning PES Done!!!"
	date '+Time: %Y-%m-%d %T'
	echo
fi
#===============================================================================
# 7 Get energy or frequency
#
if [ ${energy}x = "yesx" ]; then
	cd $PESDIR
	GenerateEnergy
fi
#===============================================================================
# Finished
#
echo
echo "All steps done!"
date '+Time: %Y-%m-%d %T'
echo
exit 0
#===============================================================================
